[TOC]

## 1. 编译模型

### 1.1 启动docker

```shell
$ cd /path/to/k230_sdk
$ docker run -u root -it --rm -v $(pwd):/mnt -v $(pwd)/toolchain:/opt/toolchain -w /mnt k230_docker
/bin/bash
```

默认挂载了两个目录

- 将k230_sdk目录挂载到docker的/mnt
- 将交叉编译工具链k230_sdk/toolchain目录挂载到docker的/opt/toolchain



### 1.2 安装wheel包

> nncase是在线安装, nncase-k230是离线安装

```shell
root@c08eb1760d50:/mnt# cd src/big/ai
root@c08eb1760d50:/mnt/src/big/ai# pip install nncase
root@c08eb1760d50:/mnt/src/big/ai# pip install x86_64/*.whl
```



### 1.3 编译

```shell
root@c08eb1760d50:/mnt/src/big/ai# cd examples/
root@c08eb1760d50:/mnt/src/big/ai/examples# ./build_model.sh
```



### 1.4 查看编译结果

```shell
root@c08eb1760d50:/mnt/src/big/ai/examples# ls -l tmp/
total 32
drwxr-xr-x 19 root root 4096 May  5 14:28 conv2d_caffe
drwxr-xr-x 19 root root 4096 May  5 14:31 mbface_depth
drwxr-xr-x 19 root root 4096 May  5 14:32 mbface_ir
drwxr-xr-x 19 root root 4096 May  5 14:31 mbv2_depth
drwxr-xr-x 19 root root 4096 May  5 14:31 mbv2_ir
drwxr-xr-x 19 root root 4096 May  5 14:28 mbv2_tflite
drwxr-xr-x 19 root root 4096 May  5 14:31 mobile_retinaface
drwxr-xr-x 19 root root 4096 May  5 14:29 yolov5s_onnx
```





## 2. 编译App

### 2.1 编译

```shell
root@c08eb1760d50:/mnt/src/big/ai/examples# ./build_app.sh
```



### 2.2 查看编译结果

编译app结束后, 默认会将三个demo及其运行所需文件拷贝到当前目录下的k230_bin子目录

```shell
root@c08eb1760d50:/mnt/src/big/ai/examples# ls -l k230_bin/
total 16
drwxr-xr-x 7 1000 1000 4096 May  6 21:37 door_lock
drwxrwxr-x 2 1000 1000 4096 May  6 21:37 face_detect
drwxr-xr-x 2 1000 1000 4096 May  6 21:37 image_classify
drwxr-xr-x 2 1000 1000 4096 May  6 21:37 object_detect
```


### 2.3 传送demo

- 将k230_bin目录下demo压缩为zip包,  放到本地的tftp目录下. 
- 在小核串口通过tftp传输到/sharefs(需提前配置好sharefs环境并保证有足够空间)
- 在大核串口下进入/sharefs执行相应demo.



## 3. 上板运行

### 3.1 图像分类

```shell
msh /sharefs/k230_bin/image_classify>ls
Directory /sharefs/k230_bin/image_classify:
labels_1001.txt     10484                    
cat.png             123126                   
..                  <DIR>                    
.                   <DIR>                    
test.kmodel         4077152                  
cat_1_3_224_224.bin 150528                   
cpp.sh              68                       
image_classify.elf  5982008      

msh /sharefs/k230_bin/image_classify>./cpp.sh
msh /sharefs/k230_bin/image_classify>case ./image_classify.elf built at May  5 2023 14:33:48
interp.run() took: 3.02956 ms
image classify result: tabby(0.343611)
```



### 3.2 目标检测

```shell
msh /sharefs/k230_bin/door_lock>ls
Directory /sharefs/k230_bin/door_lock:
mobile_retinaface   <DIR>                    
mbface_ir           <DIR>                    
mbv2_ir             <DIR>                    
mbv2_depth          <DIR>                    
mbface_depth        <DIR>                    
..                  <DIR>                    
.                   <DIR>                    
door_lock.elf       2471320                  
cpp.sh              257                  

msh /sharefs/k230_bin/object_detect>./cpp.sh
msh /sharefs/k230_bin/object_detect>case ./object_detect.elf built at May  5 2023 14:33:48
od set_input took 0.413815 ms
od run took 20.102 ms
od get output took 22.6084 ms
post process took 163.839 ms
text = bicycle:0.530000
text = dog:0.480000
text = truck:0.480000
draw result took 6.44374 ms
```



### 3.3 门锁

```shell
msh /sharefs/k230_bin/door_lock>ls
Directory /sharefs/k230_bin/door_lock:
mobile_retinaface   <DIR>                    
mbface_ir           <DIR>                    
mbv2_ir             <DIR>                    
mbv2_depth          <DIR>                    
mbface_depth        <DIR>                    
..                  <DIR>                    
.                   <DIR>                    
door_lock.elf       2471320                  
cpp.sh              257              

msh /sharefs/k230_bin/door_lock>./cpp.sh
msh /sharefs/k230_bin/door_lock>case ./door_lock.elf built at May  5 2023 14:33:48
MobileRetinaface ai2d run took 1.04785 ms
MobileRetinaface cpu copy took 0.393222 ms
MobileRetinaface kpu_run took 4.80955 ms
MobileRetinaface postprocess took 0.860518 ms
Mobilenetv2Depth ai2d run took 0.674445 ms
Mobilenetv2Depth cpu copy took 0.232111 ms
Mobilenetv2Depth kpu_run took 3.01482 ms
Mobilenetv2Depth postprocess took 0.040889 ms
Mobilenetv2Ir ai2d run took 0.557482 ms
Mobilenetv2Ir cpu copy took 0.234259 ms
Mobilenetv2Ir kpu_run took 3.0133 ms
Mobilenetv2Ir postprocess took 0.040852 ms
MbfaceDepth ai2d run took 0.670778 ms
MbfaceDepth cpu copy took 0.238074 ms
MbfaceDepth kpu_run took 4.04296 ms
MbfaceDepth postprocess took 0.550741 ms
MbfaceIr ai2d run took 0.552592 ms
MbfaceIr cpu copy took 0.234741 ms
MbfaceIr kpu_run took 4.04696 ms
MbfaceIr postprocess took 0.552037 ms
```



### 3.4 ISP人脸检测

```shell
msh /sharefs/k230_bin/face_detect>ls  
Directory /sharefs/k230_bin/face_detect:
.                   <DIR>                    
..                  <DIR>                    
test.kmodel         637208                   
face_detect.elf     3671752                  
cpp.sh              41                


case ./face_detect.elf built at May  6 2023 21:37:25
MMZ vaddr is 0x0023F000  paddr is 0x181A1000
sample_vicap ...
open success 
sample_vicap ...kd_mpi_vicap_get_sensor_info
sample_vicap ...kd_mpi_vicap_set_chn_attr, buffer_size[1382400]
sample_vicap ...dwc_dsi_init
m is 295 n is 15 vco is 17 hsfreq is 96 
phy1 config done 
=== wait_for_PHY0_PWRUP() begin=== 
=== wait_for_PHY1_PWRUP() begin=== 
attr->lan_num is 3 attr->lp_div is 8 ttr->cmd_mode is 0 
lbcc is f   
lbcc is f   
lbcc is 3a2   
VO_LAYER0_PINGPANG_SEL_MODE reg val is 0 
display_remap->w_start is 2e display_remap->h_start is 9 
sample_vicap ...kd_mpi_vicap_init
kd_vi_set_flash_mode, first_frame(0), glitch_filter(0)
sample_vicap ...kd_mpi_vicap_start_stream
Please Press Enter to Start KPU

```

回车后, 开始运行. 当检测到人脸后, 串口会打印出人脸坐标并在屏幕画框.

```shell
kd_mpi_isp_dump_frame, phys_addr[18BBD800]
box_num = 1, landmark_num = 1
face_location: 337, 649, 545, 937
[337, 649, 545, 937]
kd_mpi_isp_dump_release, phys_addr[18BBD800]
```

