#include <chrono>
#include <fstream>
#include <iostream>
#include <nncase/runtime/host_runtime_tensor.h>
#include <nncase/runtime/runtime_op_utility.h>
#include "mobile_retinaface.h"
#include "mobilenetv2_depth.h"
#include "mobilenetv2_ir.h"
#include "mbface_depth.h"
#include "mbface_ir.h"

using namespace nncase;
using namespace nncase::runtime;
using namespace nncase::runtime::detail;

template<typename T>
static int softmax(const T* src, T* dst, int length)
{
    const T alpha = *std::max_element(src, src + length);
    T denominator{ 0 };

    for (int i = 0; i < length; ++i) {
        dst[i] = std::exp(src[i] - alpha);
        denominator += dst[i];
    }

    for (int i = 0; i < length; ++i) {
        dst[i] /= denominator;
    }

    return 0;
}

int main(int argc, char *argv[])
{
    std::cout << "case " << argv[0] << " built at " << __DATE__ << " " << __TIME__ << std::endl;
    if (argc != 11)
    {
        std::cerr << "Usage: " << argv[0] << " <retinaceface_kmodel> <retinaceface_input> " \
            "<mbv2_depth_kmodel> <mbv2_depth_input> <mbv2_ir_kmodel> <mbv2_ir_input>" \
            "<mbface_depth_kmodel> <mbface_depth_input>" "<mbface_ir_kmodel> <mbface_ir_input>"<< std::endl;
        return -1;
    }

    MobileRetinaface retinaface(argv[1]);
    std::vector<std::string> retinaface_inputs = { argv[2] };
    retinaface.run(retinaface_inputs);

    Mobilenetv2Depth md(argv[3]);
    std::vector<std::string> md_inputs = { argv[4] };
    md.run(md_inputs);

    Mobilenetv2Ir mi(argv[5]);
    std::vector<std::string> mi_inputs = { argv[6] };
    mi.run(mi_inputs);

    MbfaceDepth mfd(argv[7]);
    std::vector<std::string> mfd_inputs = { argv[8] };
    mfd.run(mfd_inputs);

    MbfaceIr mfi(argv[9]);
    std::vector<std::string> mfi_inputs = { argv[10] };
    mfi.run(mfi_inputs);

    return 0;
}