﻿/* Copyright 2019-2021 Canaan Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#pragma once
#include "compiler_defs.h"
#include <nncase/ir/ops/k230/ir_types.h>
#include <string>

BEGIN_NS_NNCASE_RT_K230

enum gnne_reg_id
{
    gnne_reg_gpr0 = 0,
    gnne_reg_gpr1 = 1,
    gnne_reg_host_mem_base_addr0 = 33,
    gnne_reg_host_mem_base_addr1 = 34,
    gnne_reg_host_mem_base_addr2 = 35,
    gnne_reg_host_mem_base_addr3 = 36,
};

enum class mfu_pdp_op
{
    min = 0x1, /* 1:取最小值*/
    max = 0x0, /* 0:取最大值*/
    average = 0x2, /* 2:取平均值*/
    sum = 0x3 /* 3:求和*/
};

enum class mfu_reduce_op
{
    max = 0x0, /* 0:两者取最大值*/
    min = 0x1, /* 1:两者取最小值*/
    add = 0x2, /* 2:两者求和*/
    sub = 0x3, /* 3:左操作数减去右操作数*/
    mul = 0x4 /* 4:两者求其点积*/
};

enum class mfu_reduce_dim
{
    w = 0x0, /* 0:None*/
    hw = 0x1, /* 1:None*/
    chw = 0x2, /* 2: None*/
    nchw = 0x3, /* 3:None*/
};

enum class mfu_crop_resize
{
    biliner = 0x0, /* 0:双线性插值*/
    nearest = 0x1 /* 1:最邻近插值*/
};

enum class mfu_crop_align
{
    none = 0x0, /* 0:tensorflow align_corner=false*/
    corner = 0x1, /* 1:tensorflow align_corner=True*/
    center = 0x2 /* 2:open cv，中心对齐*/
};

enum class mn_round_mode
{
    round = 0,
    floor = 1,
    ceil = 2
};

static std::string to_string(mn_round_mode v)
{
    switch (v)
    {
    case mn_round_mode::round:
        return "round";
    case mn_round_mode::floor:
        return "floor";
    case mn_round_mode::ceil:
        return "ceil";
    default:
        throw std::runtime_error("unimplemented");
    }
}

enum class mn_triangle_mode
{
    sin = 0,
    cos = 1
};

static std::string to_string(mn_triangle_mode v)
{
    switch (v)
    {
    case mn_triangle_mode::sin:
        return "sin";
    case mn_triangle_mode::cos:
        return "cos";
    default:
        throw std::runtime_error("unimplemented");
    }
}

enum class mn_unary_logic_mode
{
    abs = 0,
    sign = 1,
    neg = 2
};

static std::string to_string(mn_unary_logic_mode v)
{
    switch (v)
    {
    case mn_unary_logic_mode::abs:
        return "abs";
    case mn_unary_logic_mode::sign:
        return "sign";
    case mn_unary_logic_mode::neg:
        return "neg";
    default:
        throw std::runtime_error("unimplemented");
    }
}

enum class mn_cmp_mode
{
    min = 0,
    max = 1,
    gt = 2,
    ge = 3,
    lt = 4,
    le = 5,
    eq = 6
};

static std::string to_string(mn_cmp_mode v)
{
    switch (v)
    {
    case mn_cmp_mode::min:
        return "min";
    case mn_cmp_mode::max:
        return "max";
    case mn_cmp_mode::gt:
        return "gt";
    case mn_cmp_mode::ge:
        return "ge";
    case mn_cmp_mode::lt:
        return "lt";
    case mn_cmp_mode::le:
        return "le";
    case mn_cmp_mode::eq:
        return "eq";
    default:
        throw std::runtime_error("unimplemented");
    }
}

enum class mn_sqrt_mode
{
    positive = 0, // 输入符号位直接认为是0，一定是正数
    nan = 1, // 如果是输入符号位位负的，返回NaN
    negative = 2 // 如果输入是正数，正常输出。否则输出负数
};

static std::string to_string(mn_sqrt_mode v)
{
    switch (v)
    {
    case mn_sqrt_mode::positive:
        return "positive";
    case mn_sqrt_mode::nan:
        return "nan";
    case mn_sqrt_mode::negative:
        return "negative";
    default:
        throw std::runtime_error("unimplemented");
    }
}

enum class mn_log_mode
{
    positive = 0, // 输入符号位直接认为是0，一定是正数
    nan = 1, // 如果是输入符号位位负的，返回NaN
    negative = 2 //如果输入是正数，正常输出。否则输出负数
};

static std::string to_string(mn_log_mode v)
{
    switch (v)
    {
    case mn_log_mode::positive:
        return "positive";
    case mn_log_mode::nan:
        return "nan";
    case mn_log_mode::negative:
        return "negative";
    default:
        throw std::runtime_error("unimplemented");
    }
}

enum class mn_binary_logic_islogic
{
    bitwise = 0,
    logic = 1
};

static std::string to_string(mn_binary_logic_islogic v)
{
    switch (v)
    {
    case mn_binary_logic_islogic::bitwise:
        return "bitwise";
    case mn_binary_logic_islogic::logic:
        return "logic";
    default:
        throw std::runtime_error("unimplemented");
    }
}

enum class mn_binary_logic_mode
{
    // add prefix to avoid collision with C++ keywords.
    kand = 0,
    kor = 1,
    knot = 2,
    kxor = 3
};

enum class sparsity
{
    dense = 0, /* 0:直接存储*/
    sparse = 1 /* 1:对0进行压缩编码*/
};

static std::string to_string(mn_binary_logic_mode v, mn_binary_logic_islogic l = mn_binary_logic_islogic::bitwise)
{
    switch (v)
    {
    case mn_binary_logic_mode::kand:
        return l == mn_binary_logic_islogic::bitwise ? "&" : "&&";
    case mn_binary_logic_mode::kor:
        return l == mn_binary_logic_islogic::bitwise ? "|" : "||";
    case mn_binary_logic_mode::knot:
        return l == mn_binary_logic_islogic::bitwise ? "~" : "!";
    case mn_binary_logic_mode::kxor:
        return l == mn_binary_logic_islogic::bitwise ? "^" : "^^";
    default:
        throw std::runtime_error("unimplemented");
    }
}

struct mfu_repeat
{
    uint32_t left_repeats;
    uint32_t scalar_repeat;
    uint32_t slice_len;
    uint32_t slice_repeat;

    bool operator==(const mfu_repeat &other) const
    {
        return left_repeats == other.left_repeats && scalar_repeat == other.scalar_repeat && slice_len == other.slice_len && slice_repeat == other.slice_repeat;
    }
};

using namespace nncase::ir;
inline shape_t align_broadcast_shape(const shape_t &src, const shape_t &dest)
{
    assert(src.size() == dest.size());
    auto it = std::find_if(src.begin(), src.end(), [](size_t dim) { return dim != 1; });
    if (it == src.end())
    {
        return shape_t(dest.size(), 1);
    }
    else
    {
        auto begin = std::distance(src.begin(), it);
        auto sub_len = src.size() - begin;
        for (int32_t i = dest.size() - sub_len; i >= 0; i--)
        {
            bool found = true;
            for (size_t j = 0; j < sub_len; j++)
            {
                auto dim = src.at(begin + j);
                if (dim != 1 && dim != dest.at(i + j))
                {
                    found = false;
                    break;
                }
            }

            if (found)
            {
                shape_t ret(i, 1);
                for (size_t j = 0; j < sub_len; j++)
                    ret.push_back(src.at(begin + j));
                auto surfix = dest.size() - ret.size();
                for (size_t j = 0; j < surfix; j++)
                    ret.push_back(1);
                return ret;
            }
        }
    }

    throw std::runtime_error("Unexpected broadcast");
}

inline mfu_repeat get_mfu_repeat(const shape_t &src, const shape_t &dest)
{
    if (std::all_of(src.begin(), src.end(), [](int32_t i) { return i == 0; }))
        return { 1, 1, 1, 1 };

    if ((src[2] != 1 && src[2] != dest[2]) or (src[3] != 1 && src[3] != dest[3]))
        return { 1, static_cast<uint32_t>(dest[3] / src[3]), static_cast<uint32_t>(src[3]), static_cast<uint32_t>(dest[2] / src[2]) };

    auto aligned_src = align_broadcast_shape(src, dest);
    size_t dim = 0;

    // 1. Broadcast left
    shape_t broadcast_left_dims;
    for (; dim < aligned_src.size(); dim++)
    {
        if (aligned_src[dim] == dest[dim] && aligned_src[dim] != 1)
            break;
        else if (aligned_src[dim] == 1)
            broadcast_left_dims.push_back(dest[dim]);
        else
            throw std::runtime_error("Unexpected broadcast");
    }

    // 2. Skip same dims
    for (; dim < aligned_src.size(); dim++)
    {
        if (aligned_src[dim] != dest[dim])
            break;
    }

    // 3. Broadcast middle
    shape_t broadcast_middle_dims;
    for (; dim < aligned_src.size(); dim++)
    {
        if (aligned_src[dim] == dest[dim])
            break;
        else if (aligned_src[dim] == 1)
            broadcast_middle_dims.push_back(dest[dim]);
        else
            throw std::runtime_error("Unexpected broadcast");
    }

    // 4. Slice len
    size_t slice_len = 1;
    for (; dim < aligned_src.size(); dim++)
    {
        if (aligned_src[dim] == dest[dim])
            slice_len *= aligned_src[dim];
        else if (aligned_src[dim] == 1)
            break;
        else
            throw std::runtime_error("Unexpected broadcast");
    }

    // 5. Broadcast right
    shape_t broadcast_right_dims;
    for (; dim < aligned_src.size(); dim++)
    {
        if (aligned_src[dim] == 1)
            broadcast_right_dims.push_back(dest[dim]);
        else
            throw std::runtime_error("Unexpected broadcast");
    }

    mfu_repeat repeat {};
    repeat.left_repeats = broadcast_left_dims.empty() ? 1 : xt::compute_size(broadcast_left_dims);
    repeat.scalar_repeat = broadcast_right_dims.empty() ? 1 : xt::compute_size(broadcast_right_dims);
    repeat.slice_len = (uint32_t)slice_len;
    repeat.slice_repeat = broadcast_middle_dims.empty() ? 1 : xt::compute_size(broadcast_middle_dims);

    // Optimize for load
    if (repeat.slice_len == 1)
    {
        // repeat scalar if possible
        repeat.scalar_repeat *= repeat.slice_repeat;
        repeat.slice_repeat = 1;
    }
    if (repeat.slice_repeat == 1 && repeat.slice_len == 1)
    {
        for (int32_t i = aligned_src.size() - 1; i >= 0; i--)
        {
            if (repeat.slice_len * aligned_src[i] < std::exp2(16))
                repeat.slice_len *= aligned_src[i];
            else
                break;
        }
    }

    return repeat;
}

END_NS_NNCASE_RT_K230
